import logging
import fmcapi
import time

def test__ravpn(fmc):
    logging.info("Testing RAVpn class.")

    starttime = str(int(time.time()))
    namer = f"_fmcapi_test_{starttime}"

    ravpn_policies = fmcapi.RAVpn(fmc=fmc)

    logging.info(f'Looking for all RAVPN Policies')
    all_ravpn_policies = ravpn_policies.get()
    logging.info(f'Found {len(all_ravpn_policies.get("items"))} RAVPN Policies')

    for policy in all_ravpn_policies.get('items'):
        ravpn_policy = fmcapi.RAVpn(fmc=fmc)
        ravpn_policy.id = policy.get('id')
        ravpn_policy.get(expanded=True)
        logging.info(f'Found {ravpn_policy.name} with ID of {ravpn_policy.id}')

    # POST, PUT and DELETE are valid endpoints in the api >7.2, however ravpn POST has a cycical dependency
    # with serveral other api endpoints - connectionprofiles, addressassignments, etc.
    # This is due to needing a container uuid of the RAVPN policy but also needing those objects to
    # create the RAVPN policy in the first place.

    logging.info("Testing RAVpn class done.\n")


"""Policy Services Classes."""

import logging
from .accesspolicies import AccessPolicies
from .accessrules import AccessRules
from .accessrules import Bulk
from .advancedsettings import AdvancedSettings
from .autonatrules import AutoNatRules
from .defaultactions import DefaultActions
from .endpoints import Endpoints
from .filepolicies import FilePolicies
from .ftdnatpolicies import FTDNatPolicies
from .ftds2svpns import FTDS2SVPNs
from .hitcounts import HitCounts
from .ikesettings import IKESettings
from .intrusionpolicies import IntrusionPolicies
from .ipsecsettings import IPSecSettings
from .manualnatrules import ManualNatRules
from .natrules import NatRules
from .prefilterpolicies import PreFilterPolicies
from .inheritancesettings import InheritanceSettings
from .ravpns import RAVpn
from .connectionprofiles import ConnectionProfiles
from .dynamicaccesspolicies import DynamicAccessPolicies

logging.debug("In the object_services __init__.py file.")

__all__ = [
    "AdvancedSettings",
    "IPSecSettings",
    "Endpoints",
    "FTDS2SVPNs",
    "IKESettings",
    "AccessPolicies",
    "AccessRules",
    "Bulk",
    "FilePolicies",
    "FTDNatPolicies",
    "AutoNatRules",
    "ManualNatRules",
    "NatRules",
    "IntrusionPolicies",
    "PreFilterPolicies",
    "HitCounts",
    "DefaultActions",
    "InheritanceSettings",
    "RAVpn",
    "ConnectionProfiles",
    "DynamicAccessPolicies",
]



import logging
import fmcapi


def test__application_category(fmc):
    logging.info("Testing ApplicationCategory class.")

    obj1 = fmcapi.ApplicationCategories(fmc=fmc)
    logging.info("All ApplicationCategories -- >")
    result = obj1.get()
    logging.info(result)
    logging.info(f"Total items: {len(result['items'])}")
    logging.info("\n")
    del obj1
    obj1 = fmcapi.ApplicationCategories(fmc=fmc, name="SMS tools")
    logging.info("One ApplicationCategory -- >")
    logging.info(obj1.get())
    logging.info("\n")

    logging.info("Testing ApplicationCategory class done.\n")



"""Policy Assignements Class."""

from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.policy_services.accesspolicies import AccessPolicies
from fmcapi.api_objects.device_services.devicerecords import DeviceRecords
from fmcapi.api_objects.device_ha_pair_services.ftddevicehapairs import FTDDeviceHAPairs
from fmcapi.api_objects.policy_services.ftdnatpolicies import FTDNatPolicies
import logging


class PolicyAssignments(APIClassTemplate):
    """The PolicyAssignments Object in the FMC."""

    VALID_JSON_DATA = ["id", "name", "type", "targets", "policy"]
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    REQUIRED_FOR_POST = ["targets", "policy"]
    REQUIRED_FOR_PUT = ["id", "targets", "policy"]
    URL_SUFFIX = "/assignment/policyassignments"
    FILTER_BY_NAME = True

    def __init__(self, fmc, **kwargs):
        """
        Initialize PolicyAssignments object.

        Set self.type to "PolicyAssignment", parse the kwargs, and set up the self.URL.

        :param fmc (object): FMC object
        :param **kwargs: Any other values passed during instantiation.
        :return: None
        """
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for PolicyAssignments class.")
        self.parse_kwargs(**kwargs)
        self.type = "PolicyAssignment"

    def ftd_natpolicy(self, name, devices):
        """
        Associate FTD NAT Policy with a list of devices.

        :param name: (str) Name of FTD NAT Policy.
        :param devices: (list) List of device names.
        """
        logging.debug("In ftd_natpolicy() for PolicyAssignments class.")
        targets = []
        pol1 = FTDNatPolicies(fmc=self.fmc)
        pol1.get(name=name)
        if "id" in pol1.__dict__:
            self.policy = {"type": pol1.type, "name": pol1.name, "id": pol1.id}
        else:
            logging.warning(
                f"FTD NAT Policy {name} not found.  Cannot set up PolicyAssignment."
            )
        for device in devices:
            if device["type"] == "device":
                dev1 = DeviceRecords(fmc=self.fmc)
                dev1.get(name=device["name"])
            elif device["type"] == "deviceHAPair":
                dev1 = FTDDeviceHAPairs(fmc=self.fmc)
                dev1.get(name=device["name"])
            if "id" in dev1.__dict__:
                logging.info(
                    f'Adding "{dev1.name}" to targets for this FTDNat PolicyAssignment.'
                )
                targets.append({"type": dev1.type, "id": dev1.id, "name": dev1.name})
            else:
                logging.warning(
                    f"Device/DeviceHA {device['name']} not found.  Cannot add to PolicyAssignment."
                )
        self.targets = targets

    def accesspolicy(self, name, devices):
        """
        Assign Access Control Policy to the list of devices.

        :param name: (str) Name of ACP.
        :param devices: (list) List of device names.
        """
        logging.debug("In accesspolicy() for PolicyAssignments class.")
        targets = []
        pol1 = AccessPolicies(fmc=self.fmc)
        pol1.get(name=name)
        if "id" in pol1.__dict__:
            self.policy = {"type": pol1.type, "name": pol1.name, "id": pol1.id}
        else:
            logging.warning(
                f"Access Control Policy {name} not found.  Cannot set up PolicyAssignment."
            )
        for device in devices:
            if device["type"] == "device":
                dev1 = DeviceRecords(fmc=self.fmc)
                dev1.get(name=device["name"])
            elif device["type"] == "deviceHAPair":
                dev1 = FTDDeviceHAPairs(fmc=self.fmc)
                dev1.get(name=device["name"])
            if "id" in dev1.__dict__:
                logging.info(
                    f'Adding "{dev1.name}" to targets for this Access Control Policy PolicyAssignment.'
                )
                targets.append({"type": dev1.type, "id": dev1.id, "name": dev1.name})
            else:
                logging.warning(
                    f"Device/DeviceHA {device['name']} not found.  Cannot add to PolicyAssignment."
                )
        self.targets = targets

    def delete(self):
        """DELETE method for API for PolicyAssignments not supported."""
        logging.info("DELETE method for API for PolicyAssignments not supported.")
        pass



"""FTDDeviceCluster Class."""

from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging


class FTDDeviceCluster(APIClassTemplate):
    """The FTDDeviceCluster Object in the FMC."""

    VALID_JSON_DATA = ["id", "name"]
    VALID_FOR_KWARGS = VALID_JSON_DATA + [
        "slavedevices",
        "modelType",
        "version",
        "sw_version",
        "healthStatus",
        "healthPolicy",
        "model",
        "modelNumber",
        "accessPolicy",
    ]
    URL_SUFFIX = "/deviceclusters/ftddevicecluster"

    def __init__(self, fmc, **kwargs):
        """
        Initialize FTDDeviceCluster object.

        Set self.type to "DeviceCluster" and parse the kwargs.

        :param fmc (object): FMC object
        :param **kwargs: Any other values passed during instantiation.
        :return: None
        """
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for FTDDeviceCluster class.")
        self.parse_kwargs(**kwargs)
        self.type = "DeviceCluster"

    def post(self):
        """POST method for API for FTDDeviceCluster not supported."""
        logging.info("POST method for API for FTDDeviceCluster not supported.")
        pass

    def put(self):
        """PUT method for API for FTDDeviceCluster not supported."""
        logging.info("PUT method for API for FTDDeviceCluster not supported.")
        pass

    def delete(self):
        """DELETE method for API for FTDDeviceCluster not supported."""
        logging.info("DELETE method for API for FTDDeviceCluster not supported.")
        pass



"""Methods to assist making unit_tests"""

import string
import random


def id_generator(size=6, chars=string.ascii_uppercase + string.digits):
    return "".join(random.choice(chars) for _ in range(size))
